import hashlib, binascii, getpass


class Hasher:

    def __init__(self, salt):
        self._salt = salt

    def hash(self, value):
        bin_hash = hashlib.pbkdf2_hmac('sha512', value.encode('utf-8'), self._salt, 100000)
        return binascii.hexlify(bin_hash).decode('utf-8')


class User:

    def __init__(self, name, password):
        self.name = name
        self.password = password


class UserStorage:

    def __init__(self):
        self._store = {}

    def store(self, user):
        self._store[user.name] = user

    def find(self, user_name):
        return self._store.get(user_name)


class GUI:

    def __init__(self, storage, hasher):
        self._storage = storage
        self._hasher = hasher

    def _create_user(self):
        user_name = input("user name: ")
        password = self._hasher.hash(getpass.getpass(prompt="password: "))
        user = User(user_name, password)
        self._storage.store(user)

    def _check_login(self):
        user_name = input("user name: ")
        password = self._hasher.hash(getpass.getpass(prompt="password: "))
        user = self._storage.find(user_name)
        if user is None or user.password != password:
            print("login failed")
        else:
            print("login successful")

    def main_menu(self):
        print("1. Create User\n2. Check Login\n0. Exit")
        option = int(input("what do you want to do? "))
        if option == 1:
            self._create_user()
        elif option == 2:
            self._check_login()
        if option != 0:
            self.main_menu()

if __name__ == '__main__':
    user_storage = UserStorage()
    hasher = Hasher(b"some_salt")
    GUI(user_storage, hasher).main_menu()